<?php

namespace App\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RedirectResponse;

use PhpOffice\PhpSpreadsheet\Reader\Csv;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use Symfony\Component\HttpFoundation\StreamedResponse;

use App\Service\AuthService;
use App\Entity\ClientPaymentEntity;
use App\Entity\BreedEntity;
use App\Entity\SpeciesEntity;
use App\Entity\BranchEntity;

/**
 * @Route("/client_payment")
 */
class ClientPaymentController extends AbstractController
{
    

   /**
    * @Route("/report", name="client_payment_report")
    */
   public function report(Request $request, AuthService $authService)
   {
         if(!$authService->isLoggedIn()) return $authService->redirectToLogin();
         if(!$authService->isUserHasAccesses(array('Report Payment'))) return $authService->redirectToHome();

         $page_title = ' Payment Report'; 
         return $this->render('ClientPayment/report.html.twig', [ 
               'page_title' => $page_title

         ]
         );
   }

       /**
     * @Route("/report/csv", name = "client_payment_report_csv")
     */
    public function client_paymentReportCsv(Request $request, AuthService $authService){

      ini_set('memory_limit', '2048M');
      set_time_limit(0);

      $data = $request->request->all();
      $em = $this->getDoctrine()->getManager();
      $userData = $this->get('session')->get('userData');
      $disasterType = $em->getRepository(DisasterTypeEntity::class)->find($data['disasterType']);
      $columnRange = range('A', 'S');

      $cellsData [] = ['cell' => 'A1', 'data' =>  'CLIENT'];
      $cellsData [] = ['cell' => 'B1', 'data' =>  'DATE'];
      $cellsData [] = ['cell' => 'C1', 'data' =>  'PAYMENT TYPE'];
      $cellsData [] = ['cell' => 'D1', 'data' =>  'AMOUNT'];
      $row = 1;
      $payments = $this->getDoctrine()->getManager()->getRepository(LodgePaymentEntity::class)->report($data, $this->get('session')->get('userData'));

      foreach($payments as $payment){
         $row++;

         $cellsData [] = ['cell' => 'A'  .$row, 'data' =>  $payment['client']];
         $cellsData [] = ['cell' => 'B' . $row , 'data' => $payment['date']];
         $cellsData [] = ['cell' => 'C' . $row, 'data' =>  $payment['type']];
         $cellsData [] = ['cell' => 'D' . $row, 'data' =>  $payment['amount']];
      }

      $page_title =  strtolower('client-payment-report');
      return $this->export_to_excel($columnRange, $cellsData, $page_title);
  }

   private function export_to_excel($columnRange, $cellsData, $page_title, $customStyle=array()) {


      $spreadSheet = new SpreadSheet();
      $activeSheet = $spreadSheet->getActiveSheet(0);

      foreach($cellsData as $cellData) {
          $activeSheet->getCell($cellData['cell'])->setValue($cellData['data']);
      }

      $activeSheet->getColumnDimension('A')->setAutoSize(true);

      $writer = new Xlsx($spreadSheet);

      $response =  new StreamedResponse(
          function () use ($writer) {
              $writer->save('php://output');
          }
      );

      $response->headers->set('Content-Type', 'text/vnd.ms-excel; charset=utf-8');
      $response->headers->set('Content-Disposition', 'attachment;filename="' . $page_title . '.xlsx"');
      $response->headers->set('Pragma', 'public');
      $response->headers->set('Cache-Control', 'maxage=1');

      return $response;
  }


}
