<?php

namespace App\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Knp\Snappy\Pdf;

use Symfony\Component\HttpFoundation\Response;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use Symfony\Component\HttpFoundation\StreamedResponse;


use App\Service\AuthService;
use App\Entity\LodgeEntity;
use App\Entity\LodgeServiceEntity;
use App\Entity\ServiceEntity;
use App\Entity\UserEntity;
use App\Entity\RoomEntity;
use App\Entity\SmsEntity;
use App\Entity\BranchSmsEntity;
use App\Form\LodgeForm;


/**
 * @Route("/lodge")
 */
class LodgeController extends AbstractController
{

      /**
     * @Route("/ajax_sales_info", name="lodge_ajax_sales_info")
     */
    public function ajaxSalesInfo(Request $request, AuthService $authService): JsonResponse
    {
       
        $result = [ 'success' =>  true, 'msg' => ''];

         if(!$authService->isLoggedIn()) { 
            $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';
         } else {
            $result['salesInfo'] = $this->getDoctrine()->getManager()->getRepository(LodgeEntity::class)->getSalesInfo($this->get('session')->get('userData'));

         }
         return new JsonResponse($result);
     }

      /**
       * @Route("/ajax_sales_analytics", name="lodge_ajax_sales_analytics")
      */
      public function ajaxSalesAnalytics(Request $request, AuthService $authService): JsonResponse
      {
         
         $result = [ 'success' =>  true, 'msg' => ''];

         if(!$authService->isLoggedIn()) { 
            $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';
         } else {
            $result['salesInAnalytics'] = $this->getDoctrine()->getManager()->getRepository(LodgeEntity::class)->getSalesAnalytics($request->query->all(), $this->get('session')->get('userData'));

         }

         $days = [];
         $stats = [];
 
         foreach($result['salesInAnalytics'] as $stat){
         
            array_push($days, $stat['dayMonth']);
            array_push($stats, round($stat['amount']));
         }

         $result['days'] = $days;
         $result['stats'] = $stats;
         $result['success'] = true;

         return new JsonResponse($result);
      }


       /**
    * @Route("/pet_hotel", name="lodge_pet_hotel")
    */
    public function petHotel(Request $request, AuthService $authService)
    {
       
       if(!$authService->isUserHasAccesses(array('Lodge Hotel'))) return $authService->redirectToHome();       
       if(!$authService->isLoggedIn()) return $authService->redirectToLogin();


       return $this->render('Dashboard/index.html.twig', ['javascripts' => array('/js/dashboard/index.js')] );
    }
        
   /**
    * @Route("/dashboard", name="lodge_dashboard")
    */
    public function dashboard(Request $request, AuthService $authService)
    {
       
       if(!$authService->isLoggedIn()) return $authService->redirectToLogin();
       if(!$authService->isUserHasAccesses(array('Lodge Dashboard'))) return $authService->redirectToHome();
       
       $page_title = ' Lodge Dashboard'; 
       return $this->render('Lodge/dashboard.html.twig', [ 
          'page_title' => $page_title, 
          'javascripts' => array('/js/lodge/dashboard.js') ]
       );
    }
    

   /**
    * @Route("", name="lodge_index")
    */
    public function index(Request $request, AuthService $authService)
    {
       
       if(!$authService->isLoggedIn()) return $authService->redirectToLogin();
       if(!$authService->isUserHasAccesses(array('Lodge'))) return $authService->redirectToHome();
       
       $page_title = ' Lodge'; 
       return $this->render('Lodge/index.html.twig', [ 
          'page_title' => $page_title, 
          'javascripts' => array('/plugins/datatables/jquery.dataTables.js','/js/lodge/index.js') ]
       );
    }

    /**
     * @Route("/ajax_detail", name="lodge_ajax_detail")
     */
    public function ajaxDetail(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];
       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}
       
        $em = $this->getDoctrine()->getManager();
        $id = $request->query->get('id');
        $lodge = $em->getRepository(LodgeEntity::class)->find(base64_decode($id)); 
        $formData['action'] =  'v';

        if(!$lodge){

            $result['success'] = false;
            $result['msg'] = 'Opps something went wrong please try again later';
        } else {

            $formOptions = array('action' => $formData['action'], 'branchId' => $authService->getUser()->getBranch()->getIdEncoded(), 'roomId' => $lodge->getRoom()->getIdEncoded());
            $form = $this->createForm(LodgeForm::class, $lodge, $formOptions);
         
            $formChoices = [
               'serviceStatuses' => $authService->serviceStatuses()
            ];
            $result['html'] = $this->renderView('Lodge/ajax_detail.html.twig', [
                     'page_title' => 'Lodge Detail',
                     'form' => $form->createView(),
                     'action' => $formData['action'],
                     'lodge' => $lodge,
                     'formChoicesInarray' => $formChoices,
                     'formChoices' => json_encode($formChoices),

               ]);
        }
      
       return new JsonResponse($result);
    }


    /**
     * @Route("/ajax_form", name="lodge_ajax_form")
     */
    public function ajaxForm(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];
       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}
       
       $em = $this->getDoctrine()->getManager();
       $formData = $request->request->all();

       $room = $em->getRepository(RoomEntity::class)->find($formData['roomId']);

       if(!$room){
         $result['success'] = false;
         $result['msg'] = 'Opps something went wrong please try again.';
       } else {

         $lodge = $em->getRepository(LodgeEntity::class)->findOneBy(['room' => $room, 'status' => 'Active']);
         
         $formData['action'] = isset($formData['action']) ? $formData['action'] : 'u';
       
         if(!$lodge) {
            $lodge = new LodgeEntity();
            $formData['action'] = 'n';
         }
  
         $formOptions = array('action' => $formData['action'], 'branchId' => $authService->getUser()->getBranch()->getIdEncoded(), 'roomId' => $room->getIdEncoded());
         $form = $this->createForm(LodgeForm::class, $lodge, $formOptions);
      
         $formChoices = [
            'serviceStatuses' => $authService->serviceStatuses()
         ];
         $result['html'] = $this->renderView('Lodge/ajax_form.html.twig', [
              'page_title' => ($formData['action'] == 'n' ? 'New' : 'Update') . '  Lodge',
              'form' => $form->createView(),
              'action' => $formData['action'],
              'javascripts' => array('/plugins/datatables/jquery.dataTables.js','/js/lodge/ajax_form.js'),
              'lodge' => $lodge,
              'formChoicesInarray' => $formChoices,
              'formChoices' => json_encode($formChoices),

            ]);
       }
     

       return new JsonResponse($result);
    }

    /**
     * @Route("/ajax_form_process", name="lodge_ajax_form_process")
     */
    public function ajaxFormProcess(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];

       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}

       $user = $authService->getUser();
       if($request->getMethod() == "POST"){
         $lodgeForm = $request->request->get('lodge_form');
         
         $em = $this->getDoctrine()->getManager();
         $errors = $em->getRepository(LodgeEntity::class)->validate($lodgeForm);

         if(!count($errors)){
            
            $lodge = $em->getRepository(LodgeEntity::class)->find($lodgeForm['id']);

            if(!$lodge) {
               $lodge = new LodgeEntity();
            }

     
            $formOptions = array('action' => $lodgeForm['action'], 'branchId' => $user->getBranch()->getIdEncoded(), 'roomId' => $lodgeForm['room']);
            $form = $this->createForm(LodgeForm::class, $lodge, $formOptions);
            switch($lodgeForm['action']) {
               case 'n': // new

                     $form->handleRequest($request);
      
                     if ($form->isValid()) {
                        $lodge->setStatus('Active');
                        $lodge->setTransactionNo($this->generateTransactionNo($user, $em));
                        $lodge->setRoomRate($em->getReference(RoomEntity::class,base64_decode($lodgeForm['room']))->getRatePerMinimumHour());
                        $em->persist($lodge);
                        $em->flush();

                        $this->processServices($lodgeForm, $lodge, $em, $user);
                        $room = $lodge->getRoom();
                        $roomCharge =  ($lodge->getRoom()->getRatePerMinimumHour() * $this->computeHours($lodge->getTimeIn(), $lodge->getTimeOut(), $room->getMinimumHour()));
                        $lodge->setBilledAmount($lodge->getTotalServiceCharge() + $roomCharge);
                        $lodge->getRoom()->setStatus('Occupied');
                        $em->flush();
   
                        $result['msg'] = 'Record successfully added to record.';
                        $result['id'] =  $lodge->getRoom()->getId();
                        $result['action'] =  'n';
                     } else {
      
                        $result['error'] = 'Ooops something went wrong please try again later.';
                     }
      
                     break;
      
               case 'u': // update   
               case 'co': // cancel

                     $form->handleRequest($request);
                     if ($form->isValid()) {

                        $em->persist($lodge);
                        $em->flush();
                        $this->processRemove($lodgeForm, $lodge, $em);
                        $this->processServices($lodgeForm, $lodge, $em, $user);
                      
                        $room = $lodge->getRoom();
                        $roomCharge =  ($lodge->getRoom()->getRatePerMinimumHour() * $this->computeHours($lodge->getTimeIn(), $lodge->getTimeOut(), $room->getMinimumHour()));
                        $lodge->setBilledAmount($lodge->getTotalServiceCharge() + $roomCharge);
                        $em->flush();

                        if($lodge->isIsReadyForPickUp()){
                           $smsOnReadyForPickups = $em->getRepository(SmsEntity::class)->findBy([
                              'isDeleted' => false,
                              'company' => $user->getBranch()->getCompany(),
                              'isLodgeReadyForPickUp' => true, 
                           ]);
               
                           foreach($smsOnReadyForPickups as $sms){
                              
                              $branchSms = $em->getRepository(BranchSmsEntity::class)->findOneBy(['lodge' => $lodge, 'sms' => $sms]);
                              if(!$branchSms){
            
                                 $branchSms =  new BranchSmsEntity();
                                 $branchSms->setSms($sms);
                                 $branchSms->setLodge($lodge);
                                 $branchSms->setMessage(str_replace('[pet_name]', $lodge->getClientPet()->getPetName(),str_replace('[client]', $lodge->getClientPet()->getClient()->getFullName(), $sms->getMessage())));
                                 $branchSms->setStatus('New');
                                 $branchSms->setContactNo($lodge->getClientPet()->getClient()->getContactNo());

                                 $branchSms->setBranch($user->getBranch());
                                 $em->persist($branchSms);
                                 $em->flush();
                              }
                           }
                        }
                   
                        if($lodgeForm['action'] == 'co'){
                                 
                           $lodge->setStatus('Completed');
                           $em->flush();
      
                           $lodge->getRoom()->setStatus('Available');
                           $em->flush();
         
                           $result['msg'] = 'Lodging successfully checkout.';
                           $result['action'] =  'co';
                        } else {

                           $result['msg'] = 'Record successfully updated.';
                           $result['id'] =  $lodge->getRoom()->getId();
                           $result['action'] =  'u';
                        }
            
                     } else {
                       
                        $result['error'] = 'Ooops2 something went wrong please try again later.';
                     }
      
                     break;
               case 'c': // cancel
               
                   
                     $form->handleRequest($request);
                     if ($form->isValid()) {
                           
                        $lodge->setStatus('Cancelled');
                        $em->flush();

                        $lodge->getRoom()->setStatus('Available');
                        $em->flush();
      
                        $result['msg'] = 'Lodging successfully cancelled.';
                        $result['action'] =  'c';
      
                     } 
                     break;     
               
               case 'd': // delete
                     $form->handleRequest($request);
                     if ($form->isValid()) {
                          
                        $lodge->setIsDeleted(true);
                        $em->flush();

      
                        $result['msg'] = 'Record successfully deleted.';
      
                     } else {
      
                        $result['error'] = 'Ooops 3something went wrong please try again later.';
                     }
      
                     break;
            }
        
         } else {

             $result['success'] = false;
             $result['msg'] = '';
             foreach ($errors as $error) {
                 
                 $result['msg'] .= $error;
             }
         }
     } else {

         $result['error'] = 'Ooops something went wrong please try again later.';
     }
    
       return new JsonResponse($result);
   }

   /**
    * @Route("/ajax_list", name="lodge_ajax_list")
    */
    public function ajax_listAction(Request $request, AuthService $authService) {

      $get = $request->query->all();

      $result = array(
          "draw" => intval($get['draw']),
          "recordsTotal" => 0,
          "recordsFiltered" => 0,
          "data" => array()
      );

      if($authService->isLoggedIn()) {
          $result = $this->getDoctrine()->getManager()->getRepository(LodgeEntity::class)->ajax_list($get, $this->get('session')->get('userData'));
      }

      return new JsonResponse($result);
   }

     /**
    * @Route("/trasaction_ajax_list", name="lodge_trasaction_ajax_list")
    */
    public function trasaction_ajax_listAction(Request $request, AuthService $authService) {

      $get = $request->query->all();

      $result = array(
          "draw" => intval($get['draw']),
          "recordsTotal" => 0,
          "recordsFiltered" => 0,
          "data" => array()
      );

      if($authService->isLoggedIn()) {
          $result = $this->getDoctrine()->getManager()->getRepository(LodgeEntity::class)->transaction_ajax_list($get, $this->get('session')->get('userData'));
      }

      return new JsonResponse($result);
   }

       /**
     * @Route("/invoice/pdf/{id}", name = "lodge_invoice_pdf")
     */
    public function invoicePdf(Request $request, AuthService $authService, Pdf $pdf, $id){

      ini_set('memory_limit', '2048M');

      $lodge  = $this->getDoctrine()->getManager()->getRepository(LodgeEntity::class)->find(base64_decode($id));

      $options = [
          'orientation' => 'Portrait',
          'print-media-type' =>  True
      ];


      $newContent = $this->renderView('Lodge/invoice.wkpdf.twig', array('lodge' => $lodge));

      $xml = $pdf->getOutputFromHtml($newContent,$options);

      $pdfResponse = array(
          'success' => true,
          'msg' => 'PDF was successfully generated.', 
          'pdfBase64' => base64_encode($xml)
      );
     
      $pdfContent = $pdfResponse['pdfBase64'];

       return new Response(base64_decode($pdfContent), 200, array(
          'Content-Type' => 'application/pdf',
          'Content-Disposition'   => 'attachment; filename="'.  'billong-statement' .'-' . date('m/d/Y') . '.pdf"'
      ));
  }

   private function processServices($form, $lodge, $em, $user){


      if(isset($form['lodgeService'])){
         
         foreach ($form['lodgeService'] as $key => $i) {
            if(isset($i['lodge_service_id'])){
               $lodgeService = $em->getRepository(LodgeServiceEntity::class)->find(base64_decode($i['lodge_service_id']));
            } else {

               $lodgeService =  new LodgeServiceEntity();
               
            }   

            $lodgeService->setService($em->getReference(ServiceEntity::class, $i['service']));
            $lodgeService->setUser($em->getReference(UserEntity::class, $i['staff']));
            $lodgeService->setStatus($i['status']);
            $lodgeService->setServiceRate($i['price']);
            $lodgeService->setLodge($lodge);
            $em->persist($lodgeService);
            $em->flush();

            if($lodgeService->getStatus() == 'In Progress'){
               $smsOnProcesses = $em->getRepository(SmsEntity::class)->findBy([
                  'isDeleted' => false,
                  'company' => $user->getBranch()->getCompany(),
                  'isServiceOnProcess' => true, 
               ]);
   
               foreach($smsOnProcesses as $sms){
                  
                  $branchSms = $em->getRepository(BranchSmsEntity::class)->findOneBy(['lodgeService' => $lodgeService, 'sms' => $sms]);
                  if(!$branchSms){

                     $branchSms =  new BranchSmsEntity();
                     $branchSms->setSms($sms);
                     $branchSms->setLodgeService($lodgeService);
                     $branchSms->setMessage(str_replace('[status]', $lodgeService->getStatus(), str_replace('[service]', $lodgeService->getService()->getDescription(), str_replace('[pet_name]', $lodgeService->getLodge()->getClientPet()->getPetName(),str_replace('[client]', $lodgeService->getLodge()->getClientPet()->getClient()->getFullName(), $sms->getMessage())))));
                     $branchSms->setStatus('New');
                     $branchSms->setBranch($user->getBranch());
                     $branchSms->setContactNo($lodgeService->getLodge()->getClientPet()->getClient()->getContactNo());
                     $em->persist($branchSms);
                     $em->flush();
                  }
               }
            }

            if($lodgeService->getStatus() == 'Completed'){
              
               $smsOnCompleteds = $em->getRepository(SmsEntity::class)->findBy([
                  'isDeleted' => false,
                  'company' => $user->getBranch()->getCompany(),
                  'isServiceCompleted' => true, 
                  
               ]);
   
               foreach($smsOnCompleteds as $sms){
                  $branchSms = $em->getRepository(BranchSmsEntity::class)->findOneBy(['lodgeService' => $lodgeService, 'sms' => $sms]);
                  if(!$branchSms){

                     $branchSms =  new BranchSmsEntity();
                     $branchSms->setSms($sms);
                     $branchSms->setLodgeService($lodgeService);
                     $branchSms->setMessage(str_replace('[status]', $lodgeService->getStatus(), str_replace('[service]', $lodgeService->getService()->getDescription(), str_replace('[pet_name]', $lodgeService->getLodge()->getClientPet()->getPetName(),str_replace('[client]', $lodgeService->getLodge()->getClientPet()->getClient()->getFullName(), $sms->getMessage())))));
                     $branchSms->setStatus('New');
                     $branchSms->setContactNo($lodgeService->getLodge()->getClientPet()->getClient()->getContactNo());
                     $branchSms->setBranch($user->getBranch());
                     $em->persist($branchSms);
                     $em->flush();
                  }
               }
            } 

         }
         
      }
   }

   

   private function processRemove($form, $lodge, $em){

      $serviceIds = isset($form['lodge_service_ids']) ? $form['lodge_service_ids'] : []; 

      //delete client organization
      foreach($lodge->getLodgeServices() as $lodge){
         if(!in_array($lodge->getIdEncoded(), $serviceIds)){
            $lodge->setIsDeleted(true);
            $em->flush();
         }
      } 
   }

   function computeHours($timeIn, $timeOut , $minimumHour){

      if(is_null($timeIn)) return 0;
      
      $t1 = strtotime($timeIn->format('Y-m-d h:i A'));
      $t2 = strtotime(($timeOut ? $timeOut->format('Y-m-d h:i A') : date('Y-m-d h:i A')));
      $diff =  $t2 - $t1;

      return  $t1 ? ceil($diff / (3600 * $minimumHour)) : 0;
   }

   private function generateTransactionNo($user, $em){
      $lodges = $em->getRepository(LodgeEntity::Class)->todayLodge($user);
      return 'L' . date('Ymd') .  (count( $lodges) + 1) ;
   }

 
}
