<?php

namespace App\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RedirectResponse;

use PhpOffice\PhpSpreadsheet\Reader\Csv;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use Symfony\Component\HttpFoundation\StreamedResponse;

use App\Service\AuthService;
use App\Entity\LodgeEntity;
use App\Entity\LodgeServiceEntity;
use App\Entity\ServiceEntity;
use App\Entity\UserEntity;
use App\Entity\LodgePaymentEntity;
use App\Form\LodgePaymentForm;


/**
 * @Route("/lodge_payment")
 */
class LodgePaymentController extends AbstractController
{

     /**
    * @Route("/ajax_index", name="lodge_payment_ajax_index")
    */
    public function ajaxIndex(Request $request, AuthService $authService)
    {
       
       if(!$authService->isLoggedIn()) return $authService->redirectToLogin();
       $result = [ 'success' =>  true, 'msg' => ''];
       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}
       
       $pr = $request->query->all();
       $lodge = $this->getDoctrine()->getManager()->getRepository(LodgeEntity::class)->find($pr['lodgeId']);
       
       if(!$lodge){

          $result['success'] = false;
          $result['msg'] = 'Oooops something went wrong please try again later.';
       } else {

            $page_title = ' Lodge Payment'; 
            $result['html']  = $this->renderView('LodgePayment/ajax_index.html.twig', [ 
               'page_title' => $page_title, 
               'lodgeId' => $lodge->getId(),
               'javascripts' => array('plugins/datatables/jquery.dataTables.js','/js/lodge_payment/ajax_index.js') ]
            );


       }
       
       return new JsonResponse($result);
    }

    
   /**
    * @Route("/ajax_list", name="lodge_payment_ajax_list")
    */
    public function ajax_listAction(Request $request, AuthService$authService) {

      $get = $request->query->all();

      $result = array(
          "draw" => intval($get['draw']),
          "recordsTotal" => 0,
          "recordsFiltered" => 0,
          "data" => array()
      );

      if($authService->isLoggedIn()) {
          $result = $this->getDoctrine()->getManager()->getRepository(LodgePaymentEntity::class)->ajax_list($get, $this->get('session')->get('userData'));
      }

      return new JsonResponse($result);
   }

   
    /**
     * @Route("/ajax_form", name="lodge_payment_ajax_form")
     */
    public function ajaxForm(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];
       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}
       
       $em = $this->getDoctrine()->getManager();
       $formData = $request->request->all();

       $lodge = $em->getRepository(LodgeEntity::class)->find($formData['lodgeId']);

       if(!$lodge){
         $result['success'] = false;
         $result['msg'] = 'Opps something went wrong please try again.';
       } else {

         $lodgePayment = $em->getRepository(LodgePaymentEntity::class)->find($formData['id']);
         if(!$lodgePayment) {
            $lodgePayment = new LodgePaymentEntity();
         }
  
         $formOptions = array('action' => $formData['action'], 'lodgeId' => $lodge->getIdEncoded());
         $form = $this->createForm(LodgePaymentForm::class, $lodgePayment, $formOptions);
         $result['html'] = $this->renderView('LodgePayment/ajax_form.html.twig', [
              'page_title' => ($formData['action'] == 'n' ? 'New' : 'Update') . '  Lodge Payment',
              'form' => $form->createView(),
              'action' => $formData['action'],
              'javascripts' => array('/js/lodge_payment/ajax_form.js'),
              'lodgeEntity' => $lodge

            ]);
       }
     

       return new JsonResponse($result);
    }

    /**
     * @Route("/ajax_form_process", name="lodge_payment_ajax_form_process")
     */
    public function ajaxFormProcess(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];

       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}

       if($request->getMethod() == "POST"){
         $lodgePaymentForm = $request->request->get('lodge_payment_form');
         
         $em = $this->getDoctrine()->getManager();
         $errors = $em->getRepository(LodgePaymentEntity::class)->validate($lodgePaymentForm);

         if(!count($errors)){
            
            $lodgePayment = $em->getRepository(LodgePaymentEntity::class)->find($lodgePaymentForm['id']);

            if(!$lodgePayment) {
               $lodgePayment = new LodgePaymentEntity();
            }

     
            $formOptions = array('action' => $lodgePaymentForm['action'], 'lodgeId' => $lodgePaymentForm['lodge']);
            $form = $this->createForm(LodgePaymentForm::class, $lodgePayment, $formOptions);

            switch($lodgePaymentForm['action']) {
               case 'n': // new

                     $form->handleRequest($request);
      
                     if ($form->isValid()) {
                        
                        $em->persist($lodgePayment);
                        $em->flush();
   
                        $result['msg'] = 'Payment successfully added to record.';
                        $result['roomId'] =  $lodgePayment->getLodge()->getRoom()->getId();
                     } else {
      
                        $result['error'] = 'Ooops something went wrong please try again later.';
                     }
      
                     break;
      
               case 'u': // update

                     $form->handleRequest($request);


                     if ($form->isValid()) {

                        $em->persist($lodgePayment);
                        $em->flush();

                        $result['msg'] = 'Payment successfully updated.';
                        $result['roomId'] =  $lodgePayment->getLodge()->getRoom()->getId();

                     } else {
                       
                        $result['error'] = 'Ooops2 something went wrong please try again later.';
                     }
      
                     break;
      
               case 'd': // delete
                     $form->handleRequest($request);
                     if ($form->isValid()) {
                          
                        $lodgePayment->setIsDeleted(true);
                        $em->flush();
      
                        $result['msg'] = 'Record successfully deleted.';
      
                     } else {
      
                        $result['error'] = 'Ooops something went wrong please try again later.';
                     }
      
                     break;
            }
        
         } else {

             $result['success'] = false;
             $result['msg'] = '';
             foreach ($errors as $error) {
                 
                 $result['msg'] .= $error;
             }
         }
     } else {

         $result['error'] = 'Ooops something went wrong please try again later.';
     }
    
       return new JsonResponse($result);
   }

    /**
    * @Route("/report", name="lodge_payment_report")
    */
    public function report(Request $request, AuthService $authService)
    {
          if(!$authService->isLoggedIn()) return $authService->redirectToLogin();
          if(!$authService->isUserHasAccesses(array('Report Payment'))) return $authService->redirectToHome();
 
          $page_title = ' Payment Report'; 
          return $this->render('LodgePayment/report.html.twig', [ 
                'page_title' => $page_title,
                'javascripts' => array('/plugins/datatables/jquery.dataTables.js','/js/lodge_payment/report.js') 
          ]
          );
    }
 
        /**
      * @Route("/report/csv", name = "lodge_payment_report_csv")
      */
     public function lodge_paymentReportCsv(Request $request, AuthService $authService){
 
       ini_set('memory_limit', '2048M');
       set_time_limit(0);
 
       $data = $request->request->all();
       $em = $this->getDoctrine()->getManager();
       $userData = $this->get('session')->get('userData');
       $columnRange = range('A', 'S');
 
       $cellsData [] = ['cell' => 'A1', 'data' =>  'CLIENT'];
       $cellsData [] = ['cell' => 'B1', 'data' =>  'DATE'];
       $cellsData [] = ['cell' => 'C1', 'data' =>  'PAYMENT TYPE'];
       $cellsData [] = ['cell' => 'D1', 'data' =>  'AMOUNT'];
       $row = 1;
       $payments = $this->getDoctrine()->getManager()->getRepository(LodgePaymentEntity::class)->report($data, $this->get('session')->get('userData'));
 
       foreach($payments as $payment){
          $row++;
 
          $cellsData [] = ['cell' => 'A'  .$row, 'data' =>  $payment['client']];
          $cellsData [] = ['cell' => 'B' . $row , 'data' => $payment['paymentDate']];
          $cellsData [] = ['cell' => 'C' . $row, 'data' =>  $payment['paymentType']];
          $cellsData [] = ['cell' => 'D' . $row, 'data' =>  $payment['amount']];
       }
 
       $page_title =  strtolower('client-payment-report');
       return $this->export_to_excel($columnRange, $cellsData, $page_title);
   }
 
    private function export_to_excel($columnRange, $cellsData, $page_title, $customStyle=array()) {
 
 
       $spreadSheet = new SpreadSheet();
       $activeSheet = $spreadSheet->getActiveSheet(0);
 
       foreach($cellsData as $cellData) {
           $activeSheet->getCell($cellData['cell'])->setValue($cellData['data']);
       }
 
       $activeSheet->getColumnDimension('A')->setAutoSize(true);
 
       $writer = new Xlsx($spreadSheet);
 
       $response =  new StreamedResponse(
           function () use ($writer) {
               $writer->save('php://output');
           }
       );
 
       $response->headers->set('Content-Type', 'text/vnd.ms-excel; charset=utf-8');
       $response->headers->set('Content-Disposition', 'attachment;filename="' . $page_title . '.xlsx"');
       $response->headers->set('Pragma', 'public');
       $response->headers->set('Cache-Control', 'maxage=1');
 
       return $response;
   }


    
}
