<?php

namespace App\Entity;

use App\Entity\Base\BaseEntity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Form\Extension\Core\DataTransformer\DateTimeToStringTransformer;

/**
 * @ORM\Entity(repositoryClass="App\Repository\RoomRepository")
 * @ORM\Table(name="room")
 * @ORM\HasLifecycleCallbacks()
 */

class RoomEntity extends BaseEntity
{
    /**
     * @ORM\Column(type="bigint")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected $id;

    /**
     * @ORM\Column(name="code", type="string")
     */
    protected $code;

    /**
     * @ORM\Column(name="description", type="string")
     */
    protected $description;

    
    /**
     * @ORM\Column(name="room_rate", type="decimal", precision=12, scale=2, nullable=true)
     */
    protected $roomRate;

    
    /**
     * @ORM\Column(name="minimum_hour", type="integer", nullable=true)
     */
    protected $minimumHour;

    /**
     * @ORM\Column(name="rate_per_minimum_hour", type="decimal", precision=12, scale=2, nullable=true)
     */
    protected $ratePerMinimumHour;

    /**
     * @ORM\Column(name="status", type="string")
     */
    protected $status;

    /**
     * @ORM\ManyToOne(targetEntity="BranchEntity", inversedBy="rooms")
     * @ORM\JoinColumn(name="branch_id", referencedColumnName="id", nullable=true)
     */
    protected $branch;

    /**
     * @ORM\ManyToOne(targetEntity="RoomTypeEntity", inversedBy="rooms")
     * @ORM\JoinColumn(name="room_type_id", referencedColumnName="id", nullable=true)
     */
    protected $roomType;

    /**
     * @ORM\OneToMany(targetEntity="LodgeEntity", mappedBy="room", cascade={"remove"})
     */
    protected $lodges;


   
    
    public function __construct($data = null)
    {
        $this->lodges = new ArrayCollection();
    }


    /**
     * Set isDeleted
     *
     * @param boolean $isDeleted
     *
     * @return RoomEntity
     */
    public function setIsDeleted($isDeleted)
    {
        $this->isDeleted = $isDeleted;

        return $this;
    }

    /*--------------------------------------------------------------------------------------------------------*/
    /*					User Defined Setters and Getters													  */
    /*--------------------------------------------------------------------------------------------------------*/

    public function getId(): ?string
    {
        return $this->id;
    }

    public function getCode(): ?string
    {
        return $this->code;
    }

    public function setCode(string $code): self
    {
        $this->code = $code;

        return $this;
    }

    public function getDescription(): ?string
    {
        return $this->description;
    }

    public function setDescription(string $description): self
    {
        $this->description = $description;

        return $this;
    }

    public function getRoomRate(): ?string
    {
        return $this->roomRate;
    }

    public function setRoomRate(?string $roomRate): self
    {
        $this->roomRate = str_replace(',','', $roomRate);

        return $this;
    }

    public function getStatus(): ?string
    {
        return $this->status;
    }

    public function setStatus(string $status): self
    {
        $this->status = $status;

        return $this;
    }

    public function getRoomType(): ?RoomTypeEntity
    {
        return $this->roomType;
    }

    public function setRoomType(?RoomTypeEntity $roomType): self
    {
        $this->roomType = $roomType;

        return $this;
    }

    public function getBranch(): ?BranchEntity
    {
        return $this->branch;
    }

    public function setBranch(?BranchEntity $branch): self
    {
        $this->branch = $branch;

        return $this;
    }

    /**
     * @return Collection<int, LodgeEntity>
     */
    public function getLodges(): Collection
    {
        return $this->lodges;
    }

    public function addLodge(LodgeEntity $lodge): self
    {
        if (!$this->lodges->contains($lodge)) {
            $this->lodges[] = $lodge;
            $lodge->setRoom($this);
        }

        return $this;
    }

    public function removeLodge(LodgeEntity $lodge): self
    {
        if ($this->lodges->removeElement($lodge)) {
            // set the owning side to null (unless already changed)
            if ($lodge->getRoom() === $this) {
                $lodge->setRoom(null);
            }
        }

        return $this;
    }

    public function getMinimumHour()
    {
        return $this->minimumHour;
    }

    public function setMinimumHour($minimumHour): self
    {
        $this->minimumHour = $minimumHour;

        return $this;
    }

    public function getRatePerMinimumHour(): ?string
    {
        return $this->ratePerMinimumHour;
    }

    public function setRatePerMinimumHour(?string $ratePerMinimumHour): self
    {
        $this->ratePerMinimumHour = str_replace(',','', $ratePerMinimumHour);

        return $this;
    }

   
}

