<?php

namespace App\Entity;

use App\Entity\Base\BaseEntity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Form\Extension\Core\DataTransformer\DateTimeToStringTransformer;

/**
 * @ORM\Entity(repositoryClass="App\Repository\SmsRepository")
 * @ORM\Table(name="sms")
 * @ORM\HasLifecycleCallbacks()
 */

class SmsEntity extends BaseEntity
{
    /**
     * @ORM\Column(type="bigint")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected $id;

    /**
     * @ORM\Column(name="message", type="text")
     */
    protected $message;

    /**
     * @ORM\Column(name="is_lodge_ready_for_pick_up", type="boolean", nullable=true)
     */
    protected $isLodgeReadyForPickUp;

    /**
     * @ORM\Column(name="is_service_on_process", type="boolean", nullable=true)
     */
    protected $isServiceOnProcess;

    /**
     * @ORM\Column(name="is_service_completed", type="boolean", nullable=true)
     */
    protected $isServiceCompleted;

    /**
     * @ORM\ManyToOne(targetEntity="CompanyEntity", inversedBy="smss")
     * @ORM\JoinColumn(name="company_id", referencedColumnName="id", nullable=true)
     */
    protected $company;
    
    /**
     * @ORM\OneToMany(targetEntity="BranchSmsEntity", mappedBy="sms", cascade={"remove"})
     */
    protected $branchSmss;

   



    public function __construct($data = null)
    {
        $this->branchSmss = new ArrayCollection();
    }

    /*--------------------------------------------------------------------------------------------------------*/
    /*					Sms Defined Setters and Getters													  */
    /*--------------------------------------------------------------------------------------------------------*/



    /**
     * Set isDeleted
     *
     * @param boolean $isDeleted
     *
     * @return SmsEntity
     */
    public function setIsDeleted($isDeleted)
    {
        $this->isDeleted = $isDeleted;

        return $this;
    }
    /*--------------------------------------------------------------------------------------------------------*/
    /*					    Defined Setters and Getters													      */
    /*--------------------------------------------------------------------------------------------------------*/

    public function getId(): ?string
    {
        return $this->id;
    }

    public function getMessage(): ?string
    {
        return $this->message;
    }

    public function setMessage(string $message): self
    {
        $this->message = $message;

        return $this;
    }

    public function isIsLodgeReadyForPickUp(): ?bool
    {
        return $this->isLodgeReadyForPickUp;
    }

    public function setIsLodgeReadyForPickUp(?bool $isLodgeReadyForPickUp): self
    {
        $this->isLodgeReadyForPickUp = $isLodgeReadyForPickUp;

        return $this;
    }

    public function isIsServiceOnProcess(): ?bool
    {
        return $this->isServiceOnProcess;
    }

    public function setIsServiceOnProcess(?bool $isServiceOnProcess): self
    {
        $this->isServiceOnProcess = $isServiceOnProcess;

        return $this;
    }

    public function isIsServiceCompleted(): ?bool
    {
        return $this->isServiceCompleted;
    }

    public function setIsServiceCompleted(?bool $isServiceCompleted): self
    {
        $this->isServiceCompleted = $isServiceCompleted;

        return $this;
    }

    public function getCompany(): ?CompanyEntity
    {
        return $this->company;
    }

    public function setCompany(?CompanyEntity $company): self
    {
        $this->company = $company;

        return $this;
    }

    /**
     * @return Collection<int, BranchSmsEntity>
     */
    public function getBranchSmss(): Collection
    {
        return $this->branchSmss;
    }

    public function addBranchSmss(BranchSmsEntity $branchSmss): self
    {
        if (!$this->branchSmss->contains($branchSmss)) {
            $this->branchSmss[] = $branchSmss;
            $branchSmss->setSms($this);
        }

        return $this;
    }

    public function removeBranchSmss(BranchSmsEntity $branchSmss): self
    {
        if ($this->branchSmss->removeElement($branchSmss)) {
            // set the owning side to null (unless already changed)
            if ($branchSmss->getSms() === $this) {
                $branchSmss->setSms(null);
            }
        }

        return $this;
    }

   
   
}
