<?php

namespace App\Entity;

use App\Entity\Base\BaseEntity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Form\Extension\Core\DataTransformer\DateTimeToStringTransformer;

/**
 * @ORM\Entity(repositoryClass="App\Repository\SpeciesRepository")
 * @ORM\Table(name="species")
 * @ORM\HasLifecycleCallbacks()
 */

class SpeciesEntity extends BaseEntity
{
    /**
     * @ORM\Column(type="bigint")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected $id;

    /**
     * @ORM\Column(name="code", type="string")
     */
    protected $code;


    /**
     * @ORM\Column(name="description", type="string")
     */
    protected $description;

    /**
     * @ORM\ManyToOne(targetEntity="BranchEntity", inversedBy="speciess")
     * @ORM\JoinColumn(name="branch_id", referencedColumnName="id", nullable=true)
     */
    protected $branch;

    /**
     * @ORM\OneToMany(targetEntity="ClientPetEntity", mappedBy="species", cascade={"remove"})
     */
    protected $clientPets;

   
    
    public function __construct($data = null)
    {
        $this->clientPets = new ArrayCollection();
    }


    /**
     * Set isDeleted
     *
     * @param boolean $isDeleted
     *
     * @return SpeciesEntity
     */
    public function setIsDeleted($isDeleted)
    {
        $this->isDeleted = $isDeleted;

        return $this;
    }

    /*--------------------------------------------------------------------------------------------------------*/
    /*					User Defined Setters and Getters													  */
    /*--------------------------------------------------------------------------------------------------------*/

    public function getId(): ?string
    {
        return $this->id;
    }

    public function getCode(): ?string
    {
        return $this->code;
    }

    public function setCode(string $code): self
    {
        $this->code = $code;

        return $this;
    }

    public function getDescription(): ?string
    {
        return $this->description;
    }

    public function setDescription(string $description): self
    {
        $this->description = $description;

        return $this;
    }

    public function getBranch(): ?BranchEntity
    {
        return $this->branch;
    }

    public function setBranch(?BranchEntity $branch): self
    {
        $this->branch = $branch;

        return $this;
    }

    /**
     * @return Collection<int, ClientPetEntity>
     */
    public function getClientPets(): Collection
    {
        return $this->clientPets;
    }

    public function addClientPet(ClientPetEntity $clientPet): self
    {
        if (!$this->clientPets->contains($clientPet)) {
            $this->clientPets[] = $clientPet;
            $clientPet->setSpecies($this);
        }

        return $this;
    }

    public function removeClientPet(ClientPetEntity $clientPet): self
    {
        if ($this->clientPets->removeElement($clientPet)) {
            // set the owning side to null (unless already changed)
            if ($clientPet->getSpecies() === $this) {
                $clientPet->setSpecies(null);
            }
        }

        return $this;
    }

  

  
}

