<?php

namespace App\Form;

use Doctrine\Persistence\ObjectManager;
use Doctrine\ORM\EntityRepository;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\PasswordType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Doctrine\ORM\QueryBuilder;

use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\HiddenType;
use Symfony\Component\Form\Extension\Core\Type\RepeatedType;
use Symfony\Component\Form\Extension\Core\Type\EmailType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\Extension\Core\Type\DateType;
use Symfony\Component\Form\Extension\Core\Type\NumberType;
use Symfony\Component\Form\Extension\Core\Type\FileType;

use Symfony\Bridge\Doctrine\Form\Type\EntityType;



use Symfony\Component\Form\FormEvents;
use Symfony\Component\Form\FormEvent;

use App\Form\DataTransformer\DataTransformer;
use App\Form\DataTransformer\DatetimeTransformer;

use App\Entity\ClientPetEntity;
use App\Entity\BranchEntity;
use App\Entity\BreedEntity;
use App\Entity\SpeciesEntity;






class ClientPetForm extends AbstractType
{
   
    private $manager;
    private $branchId;

    public function __construct(ObjectManager $manager) {
        $this->manager = $manager;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {

        $this->branchId = $options['branchId'];

        $builder
            ->add('action', HiddenType::class, array(
                'data' => $options['action'],
                'mapped' => false,
                'attr' => array(
                    'class' => 'form-action'
                )
            ))
            ->add('id', HiddenType::class)
            ->add('pet_name', TextType::class, array(
                'label' => 'Pet Name',
                'label_attr' => array(
                    'class' => 'middle'
                ),
                'attr' => [ 'class' => 'form-control'],
                'required' => false
            ))
            ->add('breed', EntityType::class, [
                'class' => BreedEntity::class,
                'query_builder' => function (EntityRepository $er): QueryBuilder {
                    return $er->createQueryBuilder('b')
                        ->where('b.branch = :branch')
                        ->setParameter('branch' , base64_decode($this->branchId))
                        ->orderBy('b.description', 'ASC');
                },
                'choice_label' => 'description',
                'label_attr' => array(
                    'class' => 'middle'
                ),
                'attr' => [ 'class' => 'form-control'],
            ])
            ->add('species', EntityType::class, [
                'class' => SpeciesEntity::class,
                'query_builder' => function (EntityRepository $er): QueryBuilder {
                    return $er->createQueryBuilder('b')
                        ->where('b.branch = :branch')
                        ->setParameter('branch' , base64_decode($this->branchId))
                        ->orderBy('b.description', 'ASC');
                },
                'choice_label' => 'description',
                'label_attr' => array(
                    'class' => 'middle'
                ),
                'attr' => [ 'class' => 'form-control'],
            ])
            ->add('photo', FileType::class, array(
                'label' => 'Upload Photo',
                'label_attr' => array(
                    'class' => 'middle'
                ),
                'attr' => array(
                    'class' => 'form-control ',
                    'accept' => "image/png, image/gif, image/jpeg"
                ),
                'required' => false,
                'mapped' => false,
                'multiple' => true
            ))
            ->addEventListener(FormEvents::PRE_SET_DATA, function(FormEvent $event, $options) use ($builder){
               
                $form = $event->getForm();
                $data = $event->getData();
            


            });



    }

    public function getName()
    {
        return 'clientPet';
    }

    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver->setDefaults(array(
            'data_class' 	  => 'App\Entity\ClientPetEntity',
            'csrf_protection' => true,
            'csrf_field_name' => '_token',
            // a unique key to help generate the secret token
            'intention'       => 'clientEntity_intention',
            'action'          => 'n',
            'branchId'        => null,
            'breeds' => null,
            'species' => null,
            'allow_extra_fields' => true,

        ));
    }
}