<?php

namespace App\Form;

use Doctrine\Persistence\ObjectManager;
use Doctrine\ORM\EntityRepository;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\PasswordType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;

use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\HiddenType;
use Symfony\Component\Form\Extension\Core\Type\RepeatedType;
use Symfony\Component\Form\Extension\Core\Type\EmailType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\Extension\Core\Type\DateType;
use Symfony\Component\Form\Extension\Core\Type\NumberType;
use Symfony\Component\Form\Extension\Core\Type\FileType;

use Symfony\Component\Form\FormEvents;
use Symfony\Component\Form\FormEvent;
use App\Form\DataTransformer\DataTransformer;
use App\Form\DataTransformer\TimeTransformer;
use App\Entity\GroomingEntity;
use App\Entity\BranchEntity;
use App\Entity\ClientPetEntity;

class GroomingForm extends AbstractType
{
   
    private $manager;

    public function __construct(ObjectManager $manager) {
        $this->manager = $manager;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder
            ->add('action', HiddenType::class, array(
                'data' => $options['action'],
                'mapped' => false,
                'attr' => array(
                    'class' => 'form-action'
                )
            ))
            ->add('id', HiddenType::class)  
            ->add('is_ready_for_pick_up', CheckboxType::class, [
                'label'    => 'Ready for pickup',
                'required' => false,
                'label_attr' => ['class' => 'form-check-label']
            ])
            ->add('remarks', TextareaType::class, [
                'label'    => 'Remarks',
                'required' => false,
                'attr' => ['row' => 5, 'class' => 'form-control']

            ])

            ->add('photo', FileType::class, array(
                'label' => 'Upload Photo',
                'label_attr' => array(
                    'class' => 'middle'
                ),
                'attr' => array(
                    'class' => 'form-control ',
                    'accept' => "image/png, image/gif, image/jpeg"
                ),
                'required' => false,
                'mapped' => false,
                'multiple' => true
            ))
            ->add('branch', HiddenType::class, array('data' => $options['branchId']))
            ->add('client_pet', HiddenType::class)
            ->addEventListener(FormEvents::PRE_SET_DATA, function(FormEvent $event, $options) {

                $form = $event->getForm();
                $data = $event->getData();
                $cp = $data->getClientPet();

                $form
                    ->add('client_pet_desc', TextType::class, array(
                        'label' => 'Client',
                        'label_attr' => array(
                            'class' => 'middle required  col-form-label'
                        ),
                        'required' => true,
                        'attr' => array(                            
                            'class' => 'form-control ',
                        ),
                        'mapped' => false,
                        'data' => $cp ? $cp->getClient()->getFullName() . ' - ' . $cp->getPetName() : ''
                    ));
                 
            });

            $builder->get('branch')->addModelTransformer(new DataTransformer($this->manager, BranchEntity::class, true, $options['branchId']));
            $builder->get('client_pet')->addModelTransformer(new DataTransformer($this->manager, ClientPetEntity::class, false));
         
    }

    public function getName()
    {
        return 'grooming';
    }

    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver->setDefaults(array(
            'data_class' 	  => 'App\Entity\GroomingEntity',
            'csrf_protection' => true,
            'csrf_field_name' => '_token',
            // a unique key to help generate the secret token
            'intention'       => 'clientEntity_intention',
            'action'          => 'n',
            'branchId'    => null,
            'allow_extra_fields' => true,
        ));
    }
}