<?php

namespace App\Service;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use App\Entity\UserEntity;

Class AuthService {

    private $em;
    private $container;

    public function __construct(EntityManagerInterface $em, ContainerInterface $container) {

        $this->em = $em;
        $this->container = $container;
    }

    public function isLoggedIn($requestUri = true) {

        $session = $this->container->get('session');

        if($session->has('userData')) {
            return true;
        } else {
            if($requestUri) {
                $req_uri = $_SERVER['REQUEST_URI'];
                if($req_uri !== $this->container->get('router')->generate('auth_login') &&
                    $req_uri !== $this->container->get('router')->generate('auth_logout') &&
                    strpos($req_uri, 'ajax') === false) $session->set('req_uri', $req_uri);
            }
            return false;
        }
    }

    /**
     * Redirects to login page
     */
    public function redirectToLogin() {

        return new RedirectResponse($this->container->get('router')->generate('auth_login'), 302);
    }

      /**
     * Get user
     */
    public function getUser() {

        $userData = $this->container->get('session')->get('userData');
        return $this->em->getRepository(UserEntity::class)->find($userData['id']);
    }

    // Original PHP code by Chirp Internet: www.chirp.com.au
    public function better_crypt($input, $rounds = 7)
    {
        $salt = "";
        $salt_chars = array_merge(range('A','Z'), range('a','z'), range(0,9));
        for($i=0; $i < 22; $i++) {
            $salt .= $salt_chars[array_rand($salt_chars)];
        }
        return crypt($input, sprintf('$2a$%02d$', $rounds) . $salt);
    }

    /**
     * Get user types
     */
    public function getUserTypes() {

        return array(
            'Admin',
            'Worker',
            'Encoder'
        );
    }

    /**
     * Get sms types
     */
    public function getSmsTypes() {

        return array(
            'Pending - Remaining Balance'
        );
    }



    public function genders(){
        return [
            'Male' => 'Male',
            'Female' => 'Female'
        ];
    }



    public function getAccesses() {

        return array(
            array('label' => 'Dashboard', 'description' => 'Dashboard', 'children' => array(
                array('label' => 'Lodge', 'description' => 'Lodge', 'children' => [
                    array('label' => 'Hotel', 'description' => 'Lodge Hotel'),
                    array('label' => 'New', 'description' => 'Lodge New'),
                    array('label' => 'Dashboard', 'description' => 'Lodge Dashboard'),
                    array('label' => 'Lodge Payment', 'description' => 'Lodge Lodge Payment', 'children' => [
                        array('label' => 'New', 'description' => 'Lodge Lodge Payment New'),
                        array('label' => 'Update', 'description' => 'Lodge Lodge Payment Update'),
                        array('label' => 'Delete', 'description' => 'Lodge Lodge Payment Delete'),
                    ]),
                ]),
                array('label' => 'Pet', 'description' => 'Client Pet', 'children' => [
                    array('label' => 'Details', 'description' => 'Client Pet Details'),
                   
                ])
             
            )),
         
            // array('label' => 'Company', 'description' => 'Company', 'children' => array(
            //     array('label' => 'Company View', 'description' => 'Company View', 'children' => array(
            //         array('label' => 'User', 'description' => 'Company View User', 'children' => array(
            //             array('label' => 'New', 'description' => 'Company View User New'),
            //             array('label' => 'Update', 'description' => 'Company View User Update'),
            //             array('label' => 'Delete', 'description' => 'Company View User Delete'),
            //         )),
            //         array('label' => 'Branch', 'description' => 'Company View Branch', 'children' => array(
            //             array('label' => 'New', 'description' => 'Company View Branch New'),
            //             array('label' => 'Update', 'description' => 'Company View Branch Update'),
            //             array('label' => 'Delete', 'description' => 'Company View Branch Delete'),
            //         )),
            //         array('label' => 'Access', 'description' => 'Company View Access', 'children' => array(
            //             array('label' => 'Update', 'description' => 'Company View Access'),
            //             array('label' => 'Update', 'description' => 'Company View Access Form'),
            //         )),
            //         array('label' => 'Sms', 'description' => 'Company View Sms', 'children' => array(
            //             array('label' => 'New', 'description' => 'Company View Sms New'),
            //             array('label' => 'Update', 'description' => 'Company View Sms Update'),
            //             array('label' => 'Delete', 'description' => 'Company View Sms Delete'),
            //         )),
            //     ))
            // )),
            array('label' => 'Schedule', 'description' => 'Schedule', 'children' => array(
                array('label' => 'Admit', 'description' => 'Schedule Admit'),

            )),
            array('label' => 'Lodge', 'description' => 'Lodge', 'children' => array(
                array('label' => 'View', 'description' => 'Lodge View'),

            )),
            array('label' => 'Billing', 'description' => 'Billing', 'children' => array(
                array('label' => 'PDF', 'description' => 'Billing PDF', 'children' => array(
                ))
            )),
            array('label' => 'Grooming', 'description' => 'Grooming', 'children' => array(
                array('label' => 'New', 'description' => 'Grooming New'),
                array('label' => 'Update', 'description' => 'Grooming Update'),
                array('label' => 'Delete', 'description' => 'Grooming Delete'),
                array('label' => 'Dashboard', 'description' => 'Grooming Dashboard'),

                array('label' => 'Grooming Payment', 'description' => 'Grooming Grooming Payment', 'children' => [
                    array('label' => 'New', 'description' => 'Grooming Grooming Payment New'),
                    array('label' => 'Update', 'description' => 'Grooming Grooming Payment Update'),
                    array('label' => 'Delete', 'description' => 'Grooming Grooming Payment Delete'),
                ]),

            )),
            array('label' => 'Client', 'description' => 'Client', 'children' => array(
                array('label' => 'New', 'description' => 'Client New'),
                array('label' => 'Update', 'description' => 'Client Update'),
                array('label' => 'Delete', 'description' => 'Client Delete'),
                array('label' => 'Details', 'description' => 'Client Details'),
                array('label' => 'Import', 'description' => 'Client Import'),

            )),
            array('label' => 'Room', 'description' => 'Room', 'children' => array(
                array('label' => 'New', 'description' => 'Room New'),
                array('label' => 'Update', 'description' => 'Room Update'),
                array('label' => 'Delete', 'description' => 'Room Delete'),

            )),
            array('label' => 'Service', 'description' => 'Service', 'children' => array(
                array('label' => 'New', 'description' => 'Service New'),
                array('label' => 'Update', 'description' => 'Service Update'),
                array('label' => 'Delete', 'description' => 'Service Delete'),

            )),
            array('label' => 'Report', 'description' => 'Report', 'children' => array(
                array('label' => 'Payment', 'description' => 'Report Payment', 'children' => array(
                    array('label' => 'CSV', 'description' => 'Report Payment Csv')
                )),
                array('label' => 'Client', 'description' => 'Report Client', 'children' => array(
                    array('label' => 'CSV', 'description' => 'Report Client Csv'),
                    array('label' => 'PDF', 'description' => 'Report Client Pdf')
                ),
            
                
            ))),
            array('label' => 'CMS', 'description' => 'CMS', 'children' => array(
                array('label' => 'Room Type', 'description' => 'CMS Room Type', 'children' => array(
                    array('label' => 'New', 'description' => 'CMS Room Type New'),
                    array('label' => 'Update', 'description' => 'CMS Room Type Update'),
                    array('label' => 'Delete', 'description' => 'CMS Room Type Delete'),
                )),
                array('label' => 'Service Type', 'description' => 'CMS Service Type', 'children' => array(
                    array('label' => 'New', 'description' => 'CMS Service Type New'),
                    array('label' => 'Update', 'description' => 'CMS Service Type Update'),
                    array('label' => 'Delete', 'description' => 'CMS Service Type Delete'),
                )),
                array('label' => 'Payment Type', 'description' => 'CMS Payment Type', 'children' => array(
                    array('label' => 'New', 'description' => 'CMS Payment Type New'),
                    array('label' => 'Update', 'description' => 'CMS Payment Type Update'),
                    array('label' => 'Delete', 'description' => 'CMS Payment Type Delete'),
                )),
                array('label' => 'Breed', 'description' => 'CMS Breed', 'children' => array(
                    array('label' => 'New', 'description' => 'CMS Breed New'),
                    array('label' => 'Update', 'description' => 'CMS Breed Update'),
                    array('label' => 'Delete', 'description' => 'CMS Breed Delete'),
                )),
                array('label' => 'Species', 'description' => 'CMS Species', 'children' => array(
                    array('label' => 'New', 'description' => 'CMS Species New'),
                    array('label' => 'Update', 'description' => 'CMS Species Update'),
                    array('label' => 'Delete', 'description' => 'CMS Species Delete'),
                )),
            )),
            array('label' => 'Settings', 'description' => 'Settings', 'children' => array(
                array('label' => 'Variable', 'description' => 'Settings Branch Variable', 'children' => array(
                    array('label' => 'New', 'description' => 'Settings Branch Variable New'),
                    array('label' => 'Update', 'description' => 'Settings Branch Variable Update'),
                    array('label' => 'Delete', 'description' => 'Settings Branch Variable Delete'),
                )),
            )),
        );
    }

    public function serviceStatuses(){
        return [
            'On Queue' => 'On Queue',
            'In Progress' => 'In Progress' ,
            'Completed' => 'Completed'
        ];
    }

     /**
     * Redirects to home page
     */
    public function redirectToHome() {

        $userData = $this->container->get('session')->get('userData');
        
        return new RedirectResponse($this->container->get('router')->generate('dashboard_index'), 302);
    }

     /**
     * Checks if the user has the ess
     */
    public function isUserHasAccesses($accessDescriptions, $hasErrorMsg=true, $matchCtr=false) {
        $session = $this->container->get('session');
        $userData = $session->get('userData');


        if($userData['type'] === 'Super Admin') {
            return true;
        } else {
            if($matchCtr) {
                $accessCtr = 0;
                foreach($accessDescriptions as $accessDescription) if(in_array($accessDescription, $userData['accesses'])) $accessCtr++;
                $hasAccess = count($accessDescriptions) === $accessCtr;
                if(!$hasAccess) {
                    if($hasErrorMsg) {
                        $session->getFlashBag()->set('error_messages', "You don't have the right to access the page. Please contact the administrator.");
                    }
                    return false;
                } else {
                    return true;
                }
            } else {
                foreach($accessDescriptions as $accessDescription) if(in_array($accessDescription, $userData['accesses'])) return true;
                if($hasErrorMsg) $session->getFlashBag()->set('error_messages', "You don't have the right to access the page. Please contact the administrator.");
                return false;
            }
        }
    }
    
    /**
     * getTimeago
     */
    function timeAgo( $time )
    {
        $time_difference = time() - strtotime($time);

        if( $time_difference < 1 ) { return 'less than 1 second ago'; }
        $condition = array( 12 * 30 * 24 * 60 * 60 =>  'year',
                    30 * 24 * 60 * 60       =>  'month',
                    24 * 60 * 60            =>  'day',
                    60 * 60                 =>  'hour',
                    60                      =>  'minute',
                    1                       =>  'second'
        );

        foreach( $condition as $secs => $str )
        {
            $d = $time_difference / $secs;

            if( $d >= 1 )
            {
                $t = round( $d );
                return 'about ' . $t . ' ' . $str . ( $t > 1 ? 's' : '' ) . ' ago';
            }
        }
    }
}